<?php
/**
 * $Horde: wicked/display.php,v 1.41 2007/01/02 00:42:09 jan Exp $
 *
 * Copyright 2003-2006 Tyler Colbert <tyler@colberts.us>
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

@define('WICKED_BASE', dirname(__FILE__));
require_once WICKED_BASE . '/lib/base.php';
require_once 'Horde/MIME/Magic.php';
require_once 'Horde/MIME/Viewer.php';
if (is_callable(array('Horde', 'loadConfiguration'))) {
    $result = Horde::loadConfiguration('mime_drivers.php', array('mime_drivers', 'mime_drivers_map'), 'horde');
    extract($result);
} else {
    require HORDE_BASE . '/config/mime_drivers.php';
}

$page = Page::getCurrentPage();
if (is_a($page, 'PEAR_Error')) {
    $notification->push(_("Internal error viewing requested page"),
                        'horde.error');
}

$actionID = Util::getFormData('actionID');
switch ($actionID) {
case 'lock':
    if (!$page->allows(WICKED_MODE_LOCKING)) {
        $notification->push(_("You are not allowed to lock this page"),
                            'horde.error');
        break;
    }
    $result = $page->lock();
    if (is_a($result, 'PEAR_Error')) {
        $notification->push(sprintf(_("Page failed to lock: %s"), $result->getMessage()),
                            'horde.error');
    } else {
        $notification->push(_("Page locked"), 'horde.success');
    }
    break;

case 'unlock':
    if (!$page->allows(WICKED_MODE_LOCKING)) {
        $notification->push(_("You are not allowed to unlock this page"),
                            'horde.error');
    }
    $result = $page->unlock();
    if (is_a($result, 'PEAR_Error')) {
        $notification->push(
            sprintf(_("Page failed to unlock: %s"), $result->getMessage()),
            'horde.error');
    } else {
        $notification->push(_("Page unlocked"), 'horde.success');
    }
    break;

case 'history':
    if ($page->allows(WICKED_MODE_HISTORY)) {
        /* Redirect to history page. */
        $url = Horde::applicationUrl('history.php', true);
        $url = Util::addParameter($url, 'page', $page->pageName());
        header('Location: ' . $url);
        exit;
    }
    $notification->push(_("This page does not have a history"), 'horde.error');
    break;

case 'special':
    $page->handleAction();
    break;

default:
    $wicked->logPageView($page->pageName());
}

if (!$page->allows(WICKED_MODE_DISPLAY)) {
    $notification->push(_("You don't have permission to view this page."),
                        'horde.error');
    if ($page->pageName() == 'WikiHome') {
        Horde::fatal(_("You don't have permission to view this page."), __FILE__, __LINE__);
    } else {
        header('Location: ' . Wicked::url('WikiHome', true));
    }
    exit;
}

$params = Util::getFormData('params');
$page->preDisplay(WICKED_MODE_DISPLAY, $params);

if (!isset($_SESSION['wickedSession']['history'])) {
    $_SESSION['wickedSession']['history'] = array();
}

$title = $page->pageTitle();
require WICKED_TEMPLATES . '/common-header.inc';
require WICKED_TEMPLATES . '/menu.inc';
$page->render(WICKED_MODE_DISPLAY, $params);
require $registry->get('templates', 'horde') . '/common-footer.inc';

if (is_a($page, 'StandardPage') &&
    (!isset($_SESSION['wickedSession']['history'][0]) ||
     $_SESSION['wickedSession']['history'][0] != $page->pageName())) {
    array_unshift($_SESSION['wickedSession']['history'], $page->pageName());
}
if (count($_SESSION['wickedSession']['history']) > 10) {
    array_pop($_SESSION['wickedSession']['history']);
}
