#!/usr/bin/php
<?php
/**
 * $Horde: wicked/scripts/upgrades/2005-03-03_page_id.php,v 1.7 2006/04/18 15:39:41 jan Exp $
 *
 * Copyright 2005-2006 Cronosys, LLC <http://www.cronosys.com/>
 *
 * See the enclosed file COPYING for license information (ASL).  If you
 * did not receive this file, see http://www.horde.org/licenses/asl.php.
 */

@define('AUTH_HANDLER', true);
@define('HORDE_BASE', dirname(__FILE__) . '/../../..');
@define('WICKED_BASE', dirname(__FILE__) . '/../..');

// Do CLI checks and environment setup first.
require_once HORDE_BASE . '/lib/core.php';
require_once 'Horde/CLI.php';

// Make sure no one runs this from the web.
if (!Horde_CLI::runningFromCLI()) {
    exit("Must be run from the command line\n");
}

// Load the CLI environment - make sure there's no time limit, init
// some variables, etc.
Horde_CLI::init();

require_once WICKED_BASE . '/lib/base.php';
require_once 'Horde/SQL.php';

/* Some databases, like postgres, restrict DROP operations to the owner
 * of the database.  Often times this is not the 'horde' user.  If
 * the script fails, try uncommenting the following.  Make sure to fill
 * in the proper values for your database.
 */
//$wicked->_params['phptype'] = 'pgsql';
//$wicked->_params['username'] = 'horde';
//$wicked->_params['password'] = '****';
//$wicked->_params['database'] = 'horde';
//$wicked->_params['persistant'] = false;
//$wicked->_params['hostspec'] = 'localhost';
//$wicked->_params['protocol'] = 'unix';
//$wicked->_params['charset'] = 'iso-8859-1';
//$wicked->_params['driverconfig'] = 'custom';
//$wicked->_connected = false;

/* Connect to database. */
$wicked->_connect();
$db = &$wicked->_db;

/* Decide which tables haven't yet got the page_id field. */
$actions = array();
$tables = array('wicked_pages' => $conf['storage']['params']['table'],
                'wicked_history' => $conf['storage']['params']['historytable'],
                'wicked_attachments' => $conf['storage']['params']['attachmenttable'],
                'wicked_attachment_history' => $conf['storage']['params']['attachmenthistorytable']);

foreach ($tables as $id => $name) {
    $info = $db->tableInfo($name);
    $need_page_id = true;
    $need_drop_page_name = false;
    foreach ($info as $field) {
        switch ($field['name']) {
        case 'page_id':
            $need_page_id = false;
            break;

        case 'page_name':
            if ($id == 'wicked_attachments' ||
                $id == 'wicked_attachment_history') {
                $need_drop_page_name = true;
            }
            break;
        }

    }
    if ($need_page_id || $need_drop_page_name) {
        $actions[] = $id;
        if ($need_page_id) {
            echo "Will add page_id to $name.\n";
        }
        if ($need_drop_page_name) {
            echo "Will drop page_name from $name.\n";
        }
    }
}

if (empty($actions)) {
    echo "Nothing to do.\n";
    exit;
}

echo "Working... ";

/* Add page_id field to tables. */
foreach ($actions as $action) {
    $tablename = $tables[$action];

    $indices = array();
    switch ($action) {
    case 'wicked_pages':
        $sql = sprintf("CREATE TABLE new_%s (
                page_id           INT NOT NULL,
                page_name         VARCHAR(100) NOT NULL,
                page_text         TEXT,
                page_hits         INT DEFAULT 0,
                page_locked       SMALLINT NOT NULL DEFAULT 0,
                page_majorversion SMALLINT NOT NULL,
                page_minorversion INT NOT NULL,
                version_created   INT NOT NULL,
                change_author     VARCHAR(255),
                change_log        TEXT,
                PRIMARY KEY (page_id),
                UNIQUE (page_name)
            );", $tablename);
        $fields = array('page_id', 'page_name', 'page_text', 'page_hits',
                        'page_locked', 'page_majorversion', 'page_minorversion',
                        'version_created', 'change_author', 'change_log');
        break;

    case 'wicked_history':
        $sql = sprintf("CREATE TABLE new_%s (
                page_id           INT NOT NULL,
                page_name         VARCHAR(100) NOT NULL,
                page_text         TEXT,
                page_majorversion SMALLINT NOT NULL,
                page_minorversion INT NOT NULL,
                version_created   INT NOT NULL,
                change_author     VARCHAR(255),
                change_log        TEXT,
                PRIMARY KEY (page_id, page_majorversion, page_minorversion)
            );", $tablename);
        $fields = array('page_id', 'page_name', 'page_text',
                        'page_majorversion', 'page_minorversion',
                        'version_created', 'change_author', 'change_log');
        $indices = array(sprintf('CREATE INDEX %s_name_idx ON %s (page_name)',
                                 $tablename, $tablename),
                         sprintf('CREATE INDEX %s_version_idx ON %s (page_majorversion, page_minorversion)',
                                 $tablename, $tablename));
        break;

    case 'wicked_attachments':
        $sql = sprintf("CREATE TABLE new_%s (
                page_id                 INT NOT NULL,
                attachment_name         VARCHAR(100) NOT NULL,
                attachment_hits         INT DEFAULT 0,
                attachment_majorversion SMALLINT NOT NULL,
                attachment_minorversion INT NOT NULL,
                attachment_created      INT NOT NULL,
                change_author           VARCHAR(255),
                change_log              TEXT,
                PRIMARY KEY (page_id, attachment_name)
            );", $tablename);
        $fields = array('page_id', 'attachment_name', 'attachment_hits',
                        'attachment_majorversion', 'attachment_minorversion',
                        'attachment_created', 'change_author', 'change_log');
        break;

    case 'wicked_attachment_history':
        $sql = sprintf("CREATE TABLE new_%s (
                page_id                 INT NOT NULL,
                attachment_name         VARCHAR(100) NOT NULL,
                attachment_majorversion SMALLINT NOT NULL,
                attachment_minorversion INT NOT NULL,
                attachment_created      INT NOT NULL,
                change_author           VARCHAR(255),
                change_log              TEXT,
                PRIMARY KEY (page_id, attachment_name, attachment_majorversion,
                             attachment_minorversion)
            );", $tablename);
        $fields = array('page_id', 'attachment_name',
                        'attachment_majorversion', 'attachment_minorversion',
                        'attachment_created', 'change_author', 'change_log');
        $indices = array(sprintf('CREATE INDEX %s_name_idx ON %s (page_id, attachment_name)',
                                 $tablename, $tablename),
                         sprintf('CREATE INDEX %s_version_idx ON %s (attachment_majorversion, attachment_minorversion)',
                                 $tablename, $tablename));
        break;
    }

    $result = $db->query($sql);
    if (is_a($result, 'PEAR_Error')) {
        var_dump($result);
        die(sprintf('Could not create new_%s: %s', $tablename, $result->getMessage()));
    }

    /* Copy data from old table to new table. */
    $resultult = $db->query(sprintf("SELECT * FROM %s;", $tablename));
    if (is_a($resultult, 'PEAR_Error')) {
        die(sprintf('Could not SELECT on %s: %s', $tablename,
                    $resultult->getMessage()));
    }

    $row = $resultult->fetchRow(DB_FETCHMODE_ASSOC);
    while (!empty($row) && !is_a($row, 'PEAR_Error')) {

        /* For wicked_pages, we make up new IDs for each page.  For other
         * tables, we look up the existing IDs in wicked_pages.  If we don't
         * find it, we assume it is orphaned data, and kludge through by
         * making up a new ID anyway. */
        if (empty($row['page_id'])) {
            if ($action == 'wicked_pages') {
                $row['page_id'] = $db->nextId($tablename);
                if (is_a($row['page_id'], 'PEAR_Error')) {
                    die(sprintf('Could not fetch seq on %s: %s', $tablename,
                                $row['page_id']->getMessage()));
                }
            } else {
                $sql = sprintf("SELECT page_id FROM wicked_pages WHERE page_name = %s;",
                               $db->quote($row['page_name']));
                $result = $db->query($sql);
                if (empty($result) || is_a($result, 'PEAR_Error')) {
                    $row['page_id'] = $db->nextId($tablename);
                } else {
                    $pagerow = $result->fetchRow(DB_FETCHMODE_ASSOC);
                    if (empty($pagerow) || is_a($pagerow, 'PEAR_Error')) {
                        $row['page_id'] = $db->nextId($tablename);
                    } else {
                        $row['page_id'] = $pagerow['page_id'];
                    }
                }
                $result->free();
            }
        }

        /* Remove any fields which don't appear in the new table. */
        foreach (array_keys($row) as $fldname) {
            if (!in_array($fldname, $fields)) {
                unset($row[$fldname]);
            }
        }

        $sql = sprintf('INSERT INTO new_%s %s;', $tablename,
                       Horde_SQL::insertValues($db, $row));
        $result = $db->query($sql);
        if (is_a($result, 'PEAR_Error')) {
            var_dump($result);
            die(sprintf('INSERT INTO new_%s: %s', $tablename,
                        $result->getMessage()));
        }

        $row = $resultult->fetchRow(DB_FETCHMODE_ASSOC);
    }
    if (is_a($row, 'PEAR_Error')) {
        die(sprintf('Could not fetch on %s: %s', $tablename,
                    $row->getMessage()));
    }
    $resultult->free();

    $result = $db->query(sprintf("DROP TABLE %s;", $tablename));
    if (is_a($result, 'PEAR_Error')) {
        var_dump($result);
        die(sprintf('Could not DROP %s: %s', $tablename, $result->getMessage()));
    }

    $result = $db->query(sprintf("ALTER TABLE new_%s RENAME TO %s;", $tablename,
                                 $tablename));
    if (is_a($result, 'PEAR_Error')) {
        var_dump($result);
        die(sprintf('Could not rename %s: %s', $tablename, $result->getMessage()));
    }

    if (count($indices)) {
        foreach ($indices as $index) {
            $result = $db->query($index);
            if (is_a($result, 'PEAR_Error')) {
                var_dump($result);
                die(sprintf('Creating index for %s: %s', $tablename,
                            $result->getMessage()));
            }
        }
    }
}

echo "done.\n";
exit;
